@@ -5,11 +5,11 @@
 #'
 #' @export
 #' @param input The file being uploaded, path to the file on your machine.
-#' @param output Destination for output geojson file. Defaults to current 
+#' @param output Destination for output geojson file. Defaults to current
 #' working directory, and gives a random alphanumeric file name
-#' @param encoding (character) The encoding passed to 
+#' @param encoding (character) The encoding passed to
 #' \code{\link[rgdal]{readOGR}}.  Default: CP1250
-#' @param verbose (logical) Printing of \code{\link[rgdal]{readOGR}} progress. 
+#' @param verbose (logical) Printing of \code{\link[rgdal]{readOGR}} progress.
 #' Default: \code{FALSE}
 #' @template read
 #' @examples \dontrun{
@@ -40,37 +40,38 @@
 #' ## beware, this is a long running example
 #' # url <- 'http://www.nws.noaa.gov/geodata/catalog/national/data/ci08au12.zip'
 #' # out <- file_to_geojson(input=url, method='web', output='cities')
-#' 
+#'
 #' # geojson with .json extension
-#' x <- gsub("\n", "", paste0('https://gist.githubusercontent.com/hunterowens/
-#' 25ea24e198c80c9fbcc7/raw/7fd3efda9009f902b5a991a506cea52db19ba143/
-#' wards2014.json', collapse = ""))
-#' res <- file_to_geojson(x)
-#' jsonlite::fromJSON(res)
-#' res <- file_to_geojson(x, method = "local")
-#' jsonlite::fromJSON(res)
+#' ## this doesn't work anymore, hmmm
+#' # x <- gsub("\n", "", paste0('https://gist.githubusercontent.com/hunterowens/
+#' # 25ea24e198c80c9fbcc7/raw/7fd3efda9009f902b5a991a506cea52db19ba143/
+#' # wards2014.json', collapse = ""))
+#' # res <- file_to_geojson(x)
+#' # jsonlite::fromJSON(res)
+#' # res <- file_to_geojson(x, method = "local")
+#' # jsonlite::fromJSON(res)
 #' }
 
-file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE, 
+file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
                             encoding = "CP1250", verbose = FALSE, ...) {
-  
+
   method <- match.arg(method, choices = c("web", "local"))
   if (!inherits(parse, "logical")) stop("parse must be logical", call. = FALSE)
-  
+
   input <- handle_remote(input)
   mem <- ifelse(output == ":memory:", TRUE, FALSE)
-  
+
   if (method == "web") {
     url <- "http://ogre.adc4gis.com/convert"
     tt <- httr::POST(url, body = list(upload = httr::upload_file(input)))
     if (tt$status_code > 201) {
       res <- tryCatch(
-        httr::content(tt, as = "text", encoding = "UTF-8"), 
+        httr::content(tt, as = "text", encoding = "UTF-8"),
         error = function(e) e
       )
       if (inherits(res, "error")) httr::stop_for_status(tt)
       res2 <- tryCatch(
-        jsonlite::fromJSON(res), 
+        jsonlite::fromJSON(res),
         error = function(e) e
       )
       if (inherits(res2, "error")) httr::stop_for_status(tt)
@@ -80,7 +81,11 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
     if (mem) {
       jsonlite::fromJSON(out, parse)
     } else {
-      if (output == ".") output <- basename(tempfile())
+      if (output == ".") {
+        temp <- tempfile()
+        on.exit(unlink(temp))
+        output <- basename(temp)
+      }
       fileConn <- file(paste0(output, ".geojson"))
       writeLines(out, fileConn)
       close(fileConn)
@@ -89,19 +94,24 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
     }
   } else {
     fileext <- ftype(input)
-    output <- if (output == ":memory:") {
-      tempfile()
+    if (output == ":memory:") {
+      temp <- tempfile()
+      output <- temp
+      on.exit(unlink(temp))
+      on.exit(unlink(output), add = TRUE)
     } else if (output == ".") {
-      basename(tempfile())
-    } else {
-      output
+      temp <- tempfile()
+      output <- basename(temp)
+      on.exit(unlink(temp))
+      on.exit(unlink(output), add = TRUE)
     }
+
     output <- path.expand(output)
     if (fileext == "kml") {
       my_layer <- rgdal::ogrListLayers(input)
-      x <- rgdal::readOGR(input, layer = my_layer[1], 
-                          drop_unsupported_fields = TRUE, 
-                          verbose = FALSE, stringsAsFactors = FALSE, 
+      x <- rgdal::readOGR(input, layer = my_layer[1],
+                          drop_unsupported_fields = TRUE,
+                          verbose = FALSE, stringsAsFactors = FALSE,
                           encoding = encoding, ...)
       write_ogr2(x, output)
       if (mem) {
@@ -112,7 +122,7 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
       }
     } else if (fileext == "shp") {
       x <- rgdal::readOGR(input, rgdal::ogrListLayers(input),
-                          verbose = FALSE, stringsAsFactors = FALSE, 
+                          verbose = FALSE, stringsAsFactors = FALSE,
                           encoding = encoding, ...)
       write_ogr2(x, output)
       if (mem) {
@@ -123,10 +133,10 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
       }
     } else if (fileext %in% c("geojson", "json")) {
       unlink(paste0(output, ".geojson"))
-      x <- rgdal::readOGR(input, rgdal::ogrListLayers(input), 
-                          verbose = FALSE, stringsAsFactors = FALSE, 
+      x <- rgdal::readOGR(input, rgdal::ogrListLayers(input),
+                          verbose = FALSE, stringsAsFactors = FALSE,
                           encoding = encoding, ...)
-      
+
       write_ogr2(x, output)
       if (mem) {
         from_json(output, parse)
@@ -135,7 +145,7 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
         file_ret(output)
       }
     } else {
-      stop("only .shp, .kml, .topojson, and url's are supported", 
+      stop("only .shp, .kml, .topojson, and url's are supported",
            call. = FALSE)
     }
   }
@@ -143,12 +153,12 @@ file_to_geojson <- function(input, method = "web", output = ".", parse = FALSE,
 
 write_ogr2 <- function(x, y) {
   unlink(paste0(y, ".geojson"))
-  rgdal::writeOGR(x, paste0(y, ".geojson"), basename(y), driver = "GeoJSON", 
+  rgdal::writeOGR(x, paste0(y, ".geojson"), basename(y), driver = "GeoJSON",
                   check_exists = FALSE)
 }
 
 file_at <- function(x) {
-  message(paste0("Success! File is at ", x, ".geojson")) 
+  message(paste0("Success! File is at ", x, ".geojson"))
 }
 
 file_ret <- function(x) {
@@ -156,6 +166,7 @@ file_ret <- function(x) {
 }
 
 from_json <- function(x, parse) {
+  on.exit(unlink(paste0(x, ".geojson")))
   jsonlite::fromJSON(paste0(x, ".geojson"), parse)
 }
 
